package models.scene

import cache.LocationType
import cache.definitions.Location
import cache.definitions.LocationsDefinition
import cache.definitions.OverlayDefinition
import cache.definitions.RegionDefinition
import cache.definitions.UnderlayDefinition
import controllers.worldRenderer.Constants
import controllers.worldRenderer.entities.Entity
import controllers.worldRenderer.entities.FloorDecoration
import controllers.worldRenderer.entities.GameObject
import controllers.worldRenderer.entities.TileModel
import controllers.worldRenderer.entities.TilePaint
import controllers.worldRenderer.entities.WallDecoration
import controllers.worldRenderer.entities.WallObject

class SceneRegion(val locationsDefinition: LocationsDefinition) {
    val tiles = Array(RegionDefinition.Z) { Array(RegionDefinition.X) { arrayOfNulls<SceneTile>(RegionDefinition.Y) } }
    val tileBrightness = Array(RegionDefinition.X + 1) { IntArray(RegionDefinition.Y + 1) }

    fun addTile(
        z: Int,
        x: Int,
        y: Int,
        overlayPath: Int,
        overlayRotation: Int,
        overlayTexture: Int,
        swHeight: Int,
        seHeight: Int,
        neHeight: Int,
        nwHeight: Int,
        swColor: Int,
        seColor: Int,
        neColor: Int,
        nwColor: Int,
        swColorB: Int,
        seColorB: Int,
        neColorB: Int,
        nwColorB: Int,
        underlayDefinition: UnderlayDefinition?,
        overlayDefinition: OverlayDefinition?,
        cacheTile: RegionDefinition.Tile
    ) {
        when (overlayPath) {
            0 -> {
                for (iz in z downTo 0) {
                    if (tiles[iz][x][y] == null) {
                        tiles[iz][x][y] = SceneTile(iz, x, y)
                    }
                }
                tiles[z][x][y]!!.tilePaint = TilePaint(
                    swHeight,
                    seHeight,
                    neHeight,
                    nwHeight,
                    swColor,
                    seColor,
                    neColor,
                    nwColor,
                    -1
                )
            }
            1 -> {
                for (iz in z downTo 0) {
                    if (tiles[iz][x][y] == null) {
                        tiles[iz][x][y] = SceneTile(iz, x, y)
                    }
                }
                tiles[z][x][y]!!.tilePaint = TilePaint(
                    swHeight,
                    seHeight,
                    neHeight,
                    nwHeight,
                    swColorB,
                    seColorB,
                    neColorB,
                    nwColorB,
                    overlayTexture
                )
            }
            else -> {
                for (iz in z downTo 0) {
                    if (tiles[iz][x][y] == null) {
                        tiles[iz][x][y] = SceneTile(iz, x, y)
                    }
                }
                tiles[z][x][y]!!.tileModel =
                    TileModel(
                        overlayPath,
                        overlayRotation,
                        overlayTexture,
                        x,
                        y,
                        swHeight,
                        seHeight,
                        neHeight,
                        nwHeight,
                        swColor,
                        seColor,
                        neColor,
                        nwColor,
                        swColorB,
                        seColorB,
                        neColorB,
                        nwColorB
                    )
            }
        }
        tiles[z][x][y]?.underlayDefinition = underlayDefinition
        tiles[z][x][y]?.overlayDefinition = overlayDefinition
        tiles[z][x][y]?.cacheTile = cacheTile
    }

    fun newFloorDecoration(z: Int, x: Int, y: Int, entity: Entity) {
        for (iz in z downTo 0) {
            if (tiles[iz][x][y] == null) {
                tiles[iz][x][y] = SceneTile(iz, x, y)
            }
        }
        entity.model.xOff = Constants.LOCAL_HALF_TILE_SIZE
        entity.model.yOff = Constants.LOCAL_HALF_TILE_SIZE
        val floorDecoration = FloorDecoration(entity)
        tiles[z][x][y]!!.floorDecoration = floorDecoration
    }

    fun newWallDecoration(
        z: Int,
        x: Int,
        y: Int,
        entity: Entity
    ) {
        for (iz in z downTo 0) {
            if (tiles[iz][x][y] == null) {
                tiles[iz][x][y] = SceneTile(iz, x, y)
            }
        }

        entity.model.xOff = Constants.LOCAL_HALF_TILE_SIZE
        entity.model.yOff = Constants.LOCAL_HALF_TILE_SIZE
        val wallDecoration = WallDecoration(entity)
        tiles[z][x][y]!!.wallDecoration = wallDecoration
    }

    fun newWall(
        z: Int,
        x: Int,
        y: Int,
        width: Int,
        length: Int,
        entity: Entity,
        entity2: Entity?,
        location: Location
    ) {
        for (iz in z downTo 0) {
            if (tiles[iz][x][y] == null) {
                tiles[iz][x][y] = SceneTile(iz, x, y)
            }
        }

        entity.model.xOff = width * REGION_SIZE
        entity.model.yOff = length * REGION_SIZE
        entity2?.model?.xOff = width * REGION_SIZE
        entity2?.model?.yOff = length * REGION_SIZE
        tiles[z][x][y]!!.wall = WallObject(entity, entity2, LocationType.fromId(location.type)!!)
        tiles[z][x][y]!!.locations.add(location)
    }

    fun newGameObject(
        z: Int,
        x: Int,
        y: Int,
        width: Int,
        length: Int,
        entity: Entity,
        location: Location
    ) {
        for (iz in z downTo 0) {
            if (tiles[iz][x][y] == null) {
                tiles[iz][x][y] = SceneTile(iz, x, y)
            }
        }

        entity.model.xOff = width * REGION_SIZE
        entity.model.yOff = length * REGION_SIZE
        tiles[z][x][y]!!.gameObjects.add(GameObject(entity))
        tiles[z][x][y]!!.locations.add(location)
    }
}
